@@ -346,6 +346,7 @@ static void
 gdm_config_init (void)
 {
    gint i;
+   gchar *key_string = NULL; 
 
    type_hash    = g_hash_table_new (g_str_hash, g_str_equal);
    val_hash     = g_hash_table_new (g_str_hash, g_str_equal);
@@ -511,7 +512,6 @@ gdm_config_init (void)
 
    /* custom command string values */
    GdmCustomCommands = g_new0 (GdmCustomCmd, GDM_CUSTOM_COMMAND_MAX);   
-   gchar *key_string = NULL; 
    for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
 	   /* For each possible custom command */      
 	   key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i); 
@@ -2013,10 +2013,15 @@ gdm_config_parse (void)
 
    /*
     * It is okay if the custom_cfg file is missing, then just use
-    * main configuration file.  If cfg is missing, then GDM will 
-    * use the built-in defaults found in gdm.h.
+    * main configuration file. However if main config file does 
+    * not exist we cannot continue and need to bail out.
     */
+
    cfg                      = gdm_get_default_config (&statbuf);
+
+   if (cfg == NULL)
+	   gdm_fail (_("%s: Main config file (defaults.conf) is missing. Aborting!"), "gdm_config_parse");
+
    config_file_mtime        = statbuf.st_mtime;
    custom_cfg               = gdm_get_custom_config (&statbuf);
    custom_config_file_mtime = statbuf.st_mtime;
