@@ -26,6 +26,7 @@
 #include "viciousui.h"
 
 #include "greeter.h"
+#include "greeter_item.h"
 #include "greeter_item_pam.h"
 #include "greeter_item_ulist.h"
 #include "greeter_item_timed.h"
@@ -45,6 +46,32 @@ gchar *greeter_current_user = NULL;
 gboolean require_quarter = FALSE;
 
 extern gboolean greeter_probably_login_prompt;
+extern GtkButton *gtk_ok_button;
+extern GtkButton *gtk_start_again_button;
+
+static gboolean
+greeter_item_pam_error_set (gboolean display)
+{
+  GreeterItemInfo *info;
+  GnomeCanvasItem *item;
+
+  info = greeter_lookup_id ("pam-error-logo");
+
+  if (info)
+    {
+      if (info->group_item != NULL)
+        item = GNOME_CANVAS_ITEM (info->group_item);
+      else
+        item = info->item;
+
+      if (display)
+          gnome_canvas_item_show (item);
+      else
+          gnome_canvas_item_hide (item);
+    }
+
+  return TRUE;
+}
 
 void
 greeter_item_pam_set_user (const char *user)
@@ -97,6 +124,11 @@ greeter_item_pam_login (GtkEntry *entry, GreeterItemInfo *info)
   char *tmp;
   GreeterItemInfo *error_info;
 
+  if (gtk_ok_button != NULL)
+	  gtk_widget_set_sensitive (GTK_WIDGET (gtk_ok_button), FALSE);
+  if (gtk_start_again_button != NULL)
+	  gtk_widget_set_sensitive (GTK_WIDGET (gtk_start_again_button), FALSE);
+
   greeter_ignore_buttons (TRUE);
 
   str = gtk_entry_get_text (GTK_ENTRY (entry));
@@ -129,8 +161,10 @@ greeter_item_pam_login (GtkEntry *entry, GreeterItemInfo *info)
       err_box_clear_handler = 0;
     }
   error_info = greeter_lookup_id ("pam-error");
-  if (error_info)
+  if (error_info) {
+    greeter_item_pam_error_set (FALSE);
     set_text (error_info, "");
+  }
   
   tmp = ve_locale_from_utf8 (str);
   printf ("%c%s\n", STX, tmp);
@@ -139,24 +173,38 @@ greeter_item_pam_login (GtkEntry *entry, GreeterItemInfo *info)
 }
 
 static gboolean
-key_press_event (GtkWidget *entry, GdkEventKey *event, gpointer data)
+pam_key_release_event (GtkWidget *entry, GdkEventKey *event, gpointer data)
 {
-  if ((event->keyval == GDK_Tab ||
-       event->keyval == GDK_KP_Tab) &&
-      (event->state & (GDK_CONTROL_MASK|GDK_MOD1_MASK|GDK_SHIFT_MASK)) == 0)
+  GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+
+  if (entry_info && entry_info->item &&
+      GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
+      GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
     {
-	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
-	if (entry_info && entry_info->item &&
-	    GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
-	    GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
-	{
-		GtkWidget *entry;
-		entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+       const char *login_string;
+	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
+       if ((event->keyval == GDK_Tab ||
+            event->keyval == GDK_KP_Tab) &&
+           (event->state & (GDK_CONTROL_MASK|GDK_MOD1_MASK|GDK_SHIFT_MASK)) == 0)
+          {
 		greeter_item_pam_login (GTK_ENTRY (entry), entry_info);
 		return TRUE;
-        }
-    }
+           }
 
+       if (gtk_ok_button != NULL)
+          {
+             /*
+              * Set ok button to sensitive only if there are characters in
+              * the entry field
+              */
+             login_string = gtk_entry_get_text (GTK_ENTRY (entry));
+             if (login_string != NULL && login_string[0] != '\0')
+		     gtk_widget_set_sensitive (GTK_WIDGET (gtk_ok_button), TRUE);
+             else
+		     gtk_widget_set_sensitive (GTK_WIDGET (gtk_ok_button), FALSE);
+          }
+    }
   return FALSE;
 }
 
@@ -165,13 +213,14 @@ greeter_item_pam_setup (void)
 {
   GreeterItemInfo *entry_info;
 
+  greeter_item_pam_error_set (FALSE);
+
   entry_info = greeter_lookup_id ("user-pw-entry");
   if (entry_info && entry_info->item &&
       GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
       GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
     {
-      GtkWidget *entry;
-      entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+      GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
       gtk_widget_grab_focus (entry);
 
       if ( ! DOING_GDM_DEVELOPMENT)
@@ -188,9 +237,10 @@ greeter_item_pam_setup (void)
 
       g_signal_connect (entry, "activate",
 			G_CALLBACK (greeter_item_pam_login), entry_info);
-      g_signal_connect (G_OBJECT (entry), "key_press_event",
-		        G_CALLBACK (key_press_event), NULL);
+      g_signal_connect (G_OBJECT (entry), "key_release_event",
+		        G_CALLBACK (pam_key_release_event), NULL);
     }
+
   return TRUE;
 }
 
@@ -272,10 +322,11 @@ static gboolean
 error_clear (gpointer data)
 {
 	GreeterItemInfo *error_info = data;
+	err_box_clear_handler       = 0;
 
         set_text (error_info, "");
+        greeter_item_pam_error_set (FALSE);
 
-	err_box_clear_handler = 0;
 	return FALSE;
 }
 
@@ -299,12 +350,14 @@ greeter_item_pam_error (const char *message)
       
       if (err_box_clear_handler > 0)
 	g_source_remove (err_box_clear_handler);
+
       if (strlen (message) == 0)
 	err_box_clear_handler = 0;
       else
 	err_box_clear_handler = g_timeout_add (30000,
 					       error_clear,
 					       error_info);
+      greeter_item_pam_error_set (TRUE);
     }
 }
 
